import { isValidUrl } from "./helpers/validityHelper";
import { InvalidUrlError } from "./errors";
import axios, { AxiosInstance } from "axios";
import { splitSignature } from "ethers/lib/utils";
import { generateNonce, verifySignature } from "./backend-api/backend-api";
import { isSmartWallet } from "./helpers/ethers";

let axiosInstance: AxiosInstance;
/**
 * Initialize the web3Provider by providing the URL
 * to the backend.
 * @param newBackendUrl
 * @throws InvalidUrlError
 */
export function initialize(newBackendUrl: string) {
    if (isValidUrl(newBackendUrl)) {
        axiosInstance = axios.create({
            baseURL: newBackendUrl
        });
    } else {
        throw new InvalidUrlError(newBackendUrl);
    }
}

/**
 * Authenticate the user by asking them to sign
 * a message. The web3Provider argument is used
 * to sign the authentication message.
 * This returns a JWT token which should be used
 * for subsequent calls.
 * @param account
 * @param web3Provider
 * @returns JWT Token
 */
export async function authenticateUser(account: string, web3Provider: any): Promise<string> {

        const chainId = web3Provider.network.chainId;
        const signerAddress = account;
      
        const nonce = await generateNonce(signerAddress, axiosInstance);
      
        const EIP712Domain = [
          { name: "name", type: "string" },
          { name: "version", type: "string" },
          { name: "chainId", type: "uint256" },
          { name: "verifyingContract", type: "address" },
        ];
        const domain = {
          name: "Boson Protocol",
          version: "1",
          chainId: chainId,
          verifyingContract: "0xCcCCccccCCCCcCCCCCCcCcCccCcCCCcCcccccccC",
        };
        const AuthSignature = [{ name: "value", type: "string" }];
        const message = {
          value: `Authentication message: ${nonce}`,
        };
        const data = JSON.stringify({
          types: {
            EIP712Domain,
            AuthSignature,
          },
          domain,
          primaryType: "AuthSignature",
          message,
        });
      
        const isSmartWalletAccount = isSmartWallet(web3Provider);
        const signatureLike = await web3Provider.send("eth_signTypedData_v4", [
          account,
          data,
        ]);
        const signature = await splitSignature(signatureLike);
        const jwt: string = await verifySignature(
          signerAddress,
          isSmartWalletAccount,
          domain,
          { AuthSignature },
          signature,
          axiosInstance
        );
      
        return jwt;

}